# Copyright 2014 Scopely, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import jmespath

from skew.exception import ClientError, ServerError


class Endpoint(object):
    """
    Represents an endpoint of a service.
    """

    def __init__(self, service, region, account):
        self.service = service
        self.region = region
        self.account = account
        self._endpoint = self.service.get_endpoint(self.region)

    def __repr__(self):
        return '%s:%s:%s' % (self.service, self.region, self.account)

    def call(self, op_name, query=None, **kwargs):
        """
        Make a request to this service endpoint.  The response data is
        returned from this call as native Python data structures.

        This method differs from the ``call`` method of the
        ``Operation`` object in botocore in the following ways:

          * The HTTP response is checked and if there is a 400
            or 500 response code it will raise an exception.
          * It automatically handles the pagination rather than
            relying on a separate pagination method call.
          * You can pass an optional jmespath query and this query
            will be applied to the data returned from the low-level
            call.  This allows you to tailor the returned data to be
            exactly what you want.

        :type op_name: str
        :param op_name: The name of the request you wish to make.

        :type query: str
        :param query: A jmespath query that will be applied to the
            data returned by the operation prior to returning
            it to the user.

        :type kwargs: keyword arguments
        :param kwargs: Additional keyword arguments you want to pass
            to the service when making the request.
        """
        if query:
            query = jmespath.compile(query)
        op = self.service.get_operation(op_name)
        if op.can_paginate:
            pages = op.paginate(self._endpoint, **kwargs)
            data = pages.build_full_result()
        else:
            http_response, data = op.call(self._endpoint, **kwargs)
            if http_response.status_code >= 500:
                raise ServerError(http_response.status_code, data, op_name)
            if http_response.status_code >= 400:
                raise ClientError(http_response.status_code, data, op_name)
        if query:
            data = query.search(data)
        return data



# Copyright (c) 2014 Scopely, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from skew.resources.aws import AWSResource


class HostedZone(AWSResource):

    class Meta(object):
        service = 'route53'
        type = 'hostedzone'
        enum_spec = ('ListHostedZones', 'HostedZones')
        detail_spec = ('GetHostedZone', 'id', None)
        id = 'Id'
        filter_name = None
        name = 'Name'
        date = None
        dimension = None

    @property
    def id(self):
        return self._id.split('/')[-1]


class ResourceRecordSet(AWSResource):

    class Meta(object):
        service = 'route53'
        type = 'rrset'
        enum_spec = ('ListResourceRecordSets', 'ResourceRecordSets')
        detail_spec = None
        id = 'Name'
        filter_name = None
        name = 'Name'
        date = None
        dimension = None



#!/usr/bin/env python

from setuptools import setup, find_packages

import os

requires = [
    'botocore==0.63.0',
    'jmespath>=0.4.1'
]


setup(
    name='skew',
    version=open(os.path.join('skew', '_version')).read(),
    description='Unique IDs to find and inspect cloud resources.',
    long_description=open('README.md').read(),
    author='Mitch Garnaat',
    author_email='mitch@garnaat.com',
    url='https://github.com/scopely-devops/skew',
    packages=find_packages(exclude=['tests*']),
    package_data={'skew': ['_version']},
    package_dir={'skew': 'skew'},
    install_requires=requires,
    license=open("LICENSE").read(),
    classifiers=(
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Natural Language :: English',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4'
    ),
)



# Copyright (c) 2014 Scopely, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

import logging

from skew.resources.aws import AWSResource


LOG = logging.getLogger(__name__)


class Group(AWSResource):

    class Meta(object):
        service = 'iam'
        type = 'group'
        enum_spec = ('ListGroups', 'Groups')
        detail_spec = None
        id = 'GroupName'
        name = 'GroupName'
        filter_name = None
        date = 'CreateDate'
        dimension = None

    @classmethod
    def filter(cls, resource_id, data):
        LOG.debug('%s == %s', resource_id, data)
        return resource_id == data['GroupName']


class User(AWSResource):

    class Meta(object):
        service = 'iam'
        type = 'user'
        enum_spec = ('ListUsers', 'Users')
        detail_spec = None
        id = 'UserName'
        filter_name = None
        name = 'UserName'
        date = 'CreateDate'
        dimension = None

    @classmethod
    def filter(cls, resource_id, data):
        LOG.debug('%s == %s', resource_id, data)
        return resource_id == data['UserName']
